@@ -1,5 +1,5 @@
-"""Some simple authentication examples.
-"""
+"""Some simple authentication examples."""
+from __future__ import annotations
 
 from collections import Counter
 from typing import cast
